import { TableColumnTypes, TableProps } from "../Table";
import { gray } from "../../../core/colors/colors";
import {
  ScrollTableContainer,
  ScrollTableMainContainer,
} from "./ScrollTable.styled";
import { THeadContainer } from "../Table/Table.styled";

type TableRenderType = "fixed" | "scroll";

export type ScrollColumnTypes<T> = {
  [key in TableRenderType]: TableColumnTypes<T>[];
};

export type ScrollTableProps<T> = Omit<TableProps<T>, "columns"> & {
  columns: ScrollColumnTypes<T>;
};

const ScrollTable = <T extends {}>({
  columns,
  tableData,
  isLoading = true,
}: ScrollTableProps<T>) => {
  const renderThead = (type: TableRenderType) => {
    return columns[type].map((column, i) => (
      <div key={`thead_${i}`} className={column.className || ""}>
        <span>{column.label}</span>
      </div>
    ));
  };

  const renderTbody = (type: TableRenderType) => {
    return tableData.map((row, i) => (
      <section key={i}>
        {columns[type].map((column) => {
          const columnToShow = column.Cell
            ? column.Cell(row)
            : row[column.accessKey!];

          return (
            <div key={`tbody_${i}`} className={column.className || ""}>
              <span>
                <>{columnToShow}</>
              </span>
            </div>
          );
        })}
      </section>
    ));
  };

  const renderSkeleton = (type: TableRenderType, backgroundColor: string) => {
    const loadingStyle = {
      width: "80px",
      height: "16px",
      borderRadius: "40px",
    };

    return columns[type].map((column, i) => (
      <div key={`skeleton_${i}`} className={column.className || ""}>
        <span
          style={
            typeof column.label !== "object"
              ? { ...loadingStyle, backgroundColor: backgroundColor }
              : {}
          }
        >
          {typeof column.label === "object" && column.label}
        </span>
      </div>
    ));
  };

  return (
    <ScrollTableMainContainer>
      {(Object.keys(columns) as TableRenderType[]).map((tableType) => (
        <ScrollTableContainer className={tableType}>
          <THeadContainer>
            {isLoading
              ? renderSkeleton(tableType, gray[300])
              : renderThead(tableType)}
          </THeadContainer>
          {isLoading
            ? [...Array(5)].map(() => (
                <section>{renderSkeleton(tableType, gray[200])}</section>
              ))
            : renderTbody(tableType)}
        </ScrollTableContainer>
      ))}
    </ScrollTableMainContainer>
  );
};

export default ScrollTable;
