import { THeadContainer, TBodyContainer, TableContainer } from "./Table.styled";
import { gray } from "../../../core/colors/colors";

export type TableColumnTypes<T> = {
  label: string | JSX.Element;
  accessKey?: keyof T;
  Cell?: (column: T) => JSX.Element;
  className?: string;
};

export type TableProps<T> = {
  columns: TableColumnTypes<T>[];
  tableData: T[];
  isLoading?: boolean;
};

const Table = <T extends {}>({
  columns,
  tableData,
  isLoading = true,
}: TableProps<T>) => {
  const renderThead = () => {
    return columns.map((column, i) => (
      <div key={`thead_${i}`} className={column.className || ""}>
        <span>{column.label}</span>
      </div>
    ));
  };

  const renderTbody = () => {
    return tableData.map((row, i) => (
      <section key={i}>
        {columns.map((column) => (
          <div key={`tbody_${i}`} className={column.className || ""}>
            <span>
              <>
                {column.Cell ? column.Cell(row) : row[column.accessKey!]}
                {!column.Cell && !column.accessKey && "데이터 없음"}
              </>
            </span>
          </div>
        ))}
      </section>
    ));
  };

  const renderSkeleton = (backgroundColor: string) => {
    const loadingStyle = {
      width: "80px",
      height: "16px",
      borderRadius: "40px",
    };

    return columns.map((column, i) => (
      <div key={`skeleton_${i}`} className={column.className || ""}>
        <span
          style={
            typeof column.label !== "object"
              ? { ...loadingStyle, backgroundColor: backgroundColor }
              : {}
          }
        >
          {typeof column.label === "object" && column.label}
        </span>
      </div>
    ));
  };

  return (
    <TableContainer>
      <THeadContainer>
        {isLoading ? renderSkeleton(gray[300]) : renderThead()}
      </THeadContainer>
      <TBodyContainer>
        {isLoading
          ? [...Array(5)].map(() => (
              <section>{renderSkeleton(gray[200])}</section>
            ))
          : renderTbody()}
      </TBodyContainer>
    </TableContainer>
  );
};

export default Table;
